/* filename:    margpm.g
** description: The program maximizes the posterior density of the 
**              marginal log likelihood
**              with respect to the DSGE model parameters
** created:     11/05/01
*/

/********************************************************
** Define lambda indices
*/

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

/********************************************************
* Calculate posterior density at starting values
* (verbose mode only)
********************************************************/
if _verbose;
    "Model              " $lmodel;
    "Prior              " $lprior;
    "Dataset            " $dataselstr;
    "Subsample          " $subTstr;
    "===================================";
    "";
    
    "Prior Density at Starting Value:";; priodens(para,pmean,pstdd,pshape);
    lnpy = -objfcn(invtrans(para));
    "Posterior at Starting Value:    ";; lnpy;
    "";

    if not(__output);
    "";"";
    "Optimization in progress..";
    "--------------------------";
    endif;
endif;      /** _verbose **/


/********************************************************
* Maximize the posterior density
********************************************************/

/** initial values: x0 and H0 **/
cc = 0;
x0 = invtrans(para) + cc* ( rndn(npara,1) );
H0 = eye(npara)*1E-4;

/** set optimization routine environment **/
nit  = 1000;        /** maximum number of iteration **/
crit = 1E-7;        /** tolerance **/

/** Use personal optimization procedure **/
/** translated from Chris Sims' CSMIN package **/
{fh,xh,g,H,itct,fcount,_csretcode} = csminwel(x0,H0,crit,nit,&objfcn);

/** clear screen **/
if __output;
    cls;
else;
    "";
endif;


/********************************************************
* Save parameter estimates and return code
********************************************************/

/** max to model parameters **/
paraest = trans(xh);
paraest = real(paraest);
paraest = paraest.*pmaskinv + pfix.*pmask;


/** output file name **/
oparaest = postpath $+ "\\" $+llam $+ lsamp $+ "pm";
opostval = postpath $+ "\\" $+llam $+ lsamp $+ "pmval";
oretcode = postpath $+ "\\" $+llam $+ lsamp $+ "pmret";

/** make output file **/
create fhparaest = ^oparaest with ^para_names, npara, 8;
wr = writer(fhparaest,paraest');
closeall fhparaest;

create fhretcode = ^oretcode with RETCODE, 1, 2;
wr = writer(fhretcode,_csretcode);
closeall fhretcode;


/********************************************************
* Estimation Output
********************************************************/
lnpy = marglh(paraest,msel,lambda);
lnprio  = priodens( paraest, pmean, pstdd, pshape);

/** posterior values **/
postnames = { LNPOST LNPY LNPRIO };
create fhpostval = ^opostval with ^postnames, 3, 8;
wr = writer(fhpostval,(lnpy+lnprio)~lnpy~lnprio);
closeall fhparaest;

/* Compute gradient in terms of model parameters */
gradmod = g./ diag(gradp(&invtrans,paraest));

if _verbose;
    "";
    "Model              " $lmodel;
    "Prior              " $lprior;
    "Dataset            " $dataselstr;
    "Subsample          " $subTstr;
    "===================================";
    "Posterior Mode     " (lnpy+lnprio);
    "Likelihood at Mode " lnpy;
    "Prior at Mode      " lnprio;
    "";
    "Parameter | Estimate | Start-V | Prior Mean | Prior Stdd | Gradient ";
    outmat = para_names'~paraest~trans(x0)~pmean~pstdd~gradmod;
    let mask[1,6] = 0 1 1 1 1 1;
    let fmt[6,3] = "-*.*s" 10 6
                   "*.*lf"  9 4
                   "*.*lf" 11 4
                   "*.*lf" 11 4
                   "*.*lf" 13 4
                   "*.*lf" 12 4;
    d = printfm(outmat,mask,fmt);
    "";
    "Return code:        ";; _csretcode;
    "";
endif;      /** _verbose **/


/********************************************************
* PROCEDURE: OBJFCN
* defines the objective function to minimize:
* the negative of log posterior
*********************************************************/
proc (1) = objfcn(para);
local lnpY,lnprio,obserror,obsvar,modelpara,retcode;

/** likelihood **/
modelpara = trans(para);
modelpara = modelpara.*pmaskinv + pfix.*pmask;

lnpy = marglh(modelpara,msel,lambda);

if ismiss(lnpY);
   lnpY = -1E8;
endif;

/** Evaluate the Prior distribution **/
lnprio = priodens(modelpara, pmean, pstdd, pshape);

/** minimize the negative of log posterior **/
retp(real(-lnpY-lnprio));
endp;
